//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "LConsol.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)
#pragma link "CommPort"
#pragma link "CSPIN"
#pragma resource "*.dfm"

TConsolLoggerForm *ConsolLoggerForm;

//      SECU
#define REQUEST_TO_SECU           '!'

#define   CHANGEMODE   'h'   //!< change mode (type of default packet)
#define   BOOTLOADER   'i'   //!< start boot loader

#define   TEMPER_PAR   'j'   //!< temperature parameters (coolant sensor, engine cooling etc)
#define   CARBUR_PAR   'k'   //!< carburetor's parameters
#define   IDLREG_PAR   'l'   //!< idling regulator parameters
#define   ANGLES_PAR   'm'   //!< advance angle (ign. timing) parameters
#define   FUNSET_PAR   'n'   //!< parametersrelated to set of functions (lookup tables)
#define   STARTR_PAR   'o'   //!< engine start parameters

#define   FNNAME_DAT   'p'   //!< used for transfering of names of set of functions (lookup tables)
#define   SENSOR_DAT   'q'   //!< used for transfering of sensors data

#define   ADCCOR_PAR   'r'   //!< parameters related to ADC corrections
#define   ADCRAW_DAT   's'   //!< used for transfering 'raw' values directly from ADC

#define   CKPS_PAR     't'   //!< CKP sensor parameters
#define   OP_COMP_NC   'u'   //!< used to indicate that specified (suspended) operation completed

#define   CE_ERR_CODES 'v'   //!< used for transfering of CE codes

#define   KNOCK_PAR    'w'   //!< parameters related to knock detection and knock chip

#define   CE_SAVED_ERR 'x'   //!< used for transfering of CE codes stored in the EEPROM
#define   FWINFO_DAT   'y'   //!< used for transfering information about firmware
#define   MISCEL_PAR   'z'   //!< miscellaneous parameters
#define   EDITAB_PAR   '{'   //!< used for transferring of data for realtime tables editing
#define   DBGVAR_DAT   ':'   //!< for watching of firmware variables (used for debug purposes)

#define   DIAGINP_DAT  '='   //!< diagnostics: send input values (analog & digital values)
#define   DIAGOUT_DAT  '^'   //!< diagnostics: receive output states (bits)

//-----------------------------------

//     ,
//     SECU
#define HOST_REQUEST                  '@'
#define LOGGER_ANSWER                 '!'
#define LOGGER_REQUEST                LOGGER_ANSWER

#define SECU_SENSOR_DAT               'q'
#define SET_CONTEXT                   '$'
#define GET_CONTEXT                   '#'
#define DEGUG_STEP                    '&'
#define LAST_SYMBOL_BLOCK             '\r'

//--------   ------------
#define NO_SAVE_CONTEXT_SECU         0x01 //
#define SMART_SAVE_CONTEXT_SECU      0x02 //
#define NO_WRITE_CONTEXT_TO_SECU     0x04 //
#define NO_LOGGED_NOT_WORKING_ENGINE 0x08 //
#define BINARY_LOG_FORMAT            0x10 //



//  SECU   
//@q0000192A11E900B40000000000000010
#define RPM_MAX       12200
#define RPM_MIN       0
#define RPM_STEP      (RPM_MAX-RPM_MIN)/100.0 //15.0

#define MAP_MAX       110.70
#define MAP_MIN       30.01
#define MAP_STEP      (MAP_MAX - MAP_MIN)/100.0

#define VOLT_MAX      14.84
#define VOLT_MIN      9.01
#define VOLT_STEP     (VOLT_MAX-VOLT_MIN)/100.0

#define DTOJ_MAX      110.5
#define DTOJ_MIN      -28.01
#define DTOJ_STEP     (DTOJ_MAX-DTOJ_MIN)/100.0

#define UOZ_MAX       99.87
#define UOZ_MIN       -10.01
#define UOZ_STEP      (UOZ_MAX-UOZ_MIN)/100.0

#define KNOCK_K_MAX   9.9
#define KNOCK_K_MIN   0.01
#define KNOCK_K_STEP  (KNOCK_K_MAX-KNOCK_K_MIN)/100.0


#define KNOCK_R_MAX   16.02
#define KNOCK_R_MIN   0.01
#define KNOCK_R_STEP  (KNOCK_R_MAX-KNOCK_R_MIN)/100.0

#define AIRFLOW_MAX   16
#define AIRFLOW_MIN   1
#define AIRFLOW_STEP  1

#define PACK_BYTE_MAX   15
#define PACK_BYTE_MIN   0
#define PACK_BYTE_SHIFT 1

// ----------  ASCII HEX  int ---------------
unsigned int StrToInt(unsigned char* aBuf,unsigned char aSizeof)
{
  //static
  unsigned char tTmp;
  //static
  unsigned char tIndx;
  //static
  unsigned int tValue;

  tValue = 0;
  tIndx = 0;

  aSizeof<<=1;

  do
  {
    tTmp = aBuf[tIndx] - '0';

    if (9 < tTmp)
          tTmp -= 7;
    tValue <<= 4;
    tValue |= (tTmp & 0x0f);

  }while (++tIndx < aSizeof);

  return(tValue);
}
//--------------------------------------------------------------------

//---       HEX ASCII ----
unsigned char NibblesToHex(unsigned int aNibblesValue)
{
   if(aNibblesValue <=9)
       return ( aNibblesValue+'0');

   return (aNibblesValue-0x0A)+'A';
}

unsigned char* CharToHex(unsigned char aValue, unsigned char* aBuf)
{
 for(signed char tCount = sizeof(Byte)*2 -1,
      tNibblesChar = aValue & 0x0F; tCount >=0;
       tCount--, tNibblesChar = (unsigned char)aValue >> 4)
 {

   aBuf[tCount]= NibblesToHex(tNibblesChar);

 }
 return aBuf;
}

unsigned char* IntToHex(unsigned int aValue, unsigned char* aBuf)
{
 CharToHex(aValue, &aBuf[sizeof(Byte)*2]);
 CharToHex(aValue >> 8, aBuf);
 return aBuf;
}
//-------------------------------------------------------------------

//------------    SECU --------------
void LogPackedSECU::Initialize(bool aShiftParametr)
{
   shiftParametr = aShiftParametr;
   if(shiftParametr)
   {
     rpm      = RPM_MIN;      //
     map      = MAP_MIN;      //  xxxx /64  0x192A  100,65
     volt     = VOLT_MIN;     //  xxxx /400 0x11E9  11,46
     dtoj     = DTOJ_MIN;     //  xxxx /4   0x00B4  45
     uoz      = UOZ_MIN;      //  xxxx /32  0
     knock_k  = KNOCK_K_MIN;  //  xxxx /400 0
     knock_r  = KNOCK_R_MIN;  //  xxxx /32  0
     airflow  = AIRFLOW_MIN;  //
     packByte = PACK_BYTE_MIN;//
   }
   else
   {
      //  SECU   
      //    f   m   v   t   a   k  kr af
      //@q0000192A11E900B40000000000000010
     //{"q2EE01919173101BA0C7C102A013510FF\r"};

     //    1     64   400    4   32     400   32
     //    f     m     v     t    a      k    kr  af
     //@12435,100.39,14.84,110.5,99.87,10.34,9.65,16,1,1,1,1,\r

     rpm      = RPM_MAX;      //0x2EE0
     map      = MAP_MAX;      //0x1919 /64
     volt     = VOLT_MAX;     //0x1731 /400
     dtoj     = DTOJ_MAX;     //0x01BA /4
     uoz      = UOZ_MAX;      //0x0C7C /32
     knock_k  = KNOCK_K_MAX;  //0x102A /400
     knock_r  = KNOCK_R_MAX;  //0x0135 /32
     airflow  = AIRFLOW_MAX;  //0x10
     packByte = PACK_BYTE_MAX;//0xFF
   }
   SymPackedSECU.hostRequest = HOST_REQUEST;
   SymPackedSECU.commandID = SECU_SENSOR_DAT;
   MakeSymPackedSECU();
}

char* LogPackedSECU::MakeSymPackedSECU(void)
{
  //------- RPM -------------------
  if(shiftParametr)
  {
    rpm += RPM_STEP;
    if(rpm > RPM_MAX)      //0x2EE0
     rpm = RPM_MIN;

    //------- MAP -------------------
    #define MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER  64
    map += MAP_STEP;
    if( map > MAP_MAX) //0x1919 /64
      map = MAP_MIN;


    //------- VOLT -------------------
    #define UBAT_PHYSICAL_MAGNITUDE_MULTIPLAYER 400
    volt += VOLT_STEP;

    if( volt > VOLT_MAX) //0x1731 /400
      volt = VOLT_MIN;

    //------- DTOJ -------------------
    #define TEMP_PHYSICAL_MAGNITUDE_MULTIPLAYER 4
    dtoj += DTOJ_STEP;
    if( dtoj > DTOJ_MAX) //0x01BA /4
      dtoj = DTOJ_MIN;


    //------- UOZ -------------------
    #define ANGLE_MULTIPLAYER 32
    uoz += UOZ_STEP;
    if( uoz > UOZ_MAX) //0x0C7C /32
      uoz = UOZ_MIN;


    //------- KNOCK_K -------------------
    //#define UBAT_PHYSICAL_MAGNITUDE_MULTIPLAYER 400
    knock_k += KNOCK_K_STEP;

    if( knock_k > KNOCK_K_MAX) //0x102A /400
      knock_k = KNOCK_K_MIN;

    //------- KNOCK_R -------------------
    //#define ANGLE_MULTIPLAYER 32
    knock_r += KNOCK_R_STEP;
    if( knock_r > KNOCK_R_MAX) //0x0135 /32
      knock_r = KNOCK_R_MIN;


    //------- AIRFLOW -------------------
    airflow += AIRFLOW_STEP;
    if( airflow > AIRFLOW_MAX) //0x10
      airflow = AIRFLOW_MIN;


    //------- PACK_BYTE -------------------
    packByte = PACK_BYTE_MAX;
    if( packByte > PACK_BYTE_MAX) //0x10
      packByte = PACK_BYTE_MIN;
  }//

  int tParam = rpm;
  int tCountParam = 0;
  IntToHex(tParam, &SymPackedSECU.data[tCountParam]);

  tParam = map*MAP_PHYSICAL_MAGNITUDE_MULTIPLAYER;
  tCountParam += 4;
  IntToHex(tParam, &SymPackedSECU.data[tCountParam]);

  tParam = volt*UBAT_PHYSICAL_MAGNITUDE_MULTIPLAYER;
  tCountParam += 4;
  IntToHex(tParam, &SymPackedSECU.data[tCountParam]);

  tParam = dtoj*TEMP_PHYSICAL_MAGNITUDE_MULTIPLAYER;
  tCountParam += 4;
  IntToHex(tParam, &SymPackedSECU.data[tCountParam]);

  tParam = uoz*ANGLE_MULTIPLAYER;
  tCountParam += 4;
  IntToHex(tParam, &SymPackedSECU.data[tCountParam]);

  tParam = knock_k*UBAT_PHYSICAL_MAGNITUDE_MULTIPLAYER;
  tCountParam += 4;
  IntToHex(tParam, &SymPackedSECU.data[tCountParam]);


  tParam = knock_r*ANGLE_MULTIPLAYER;
  tCountParam += 4;
  IntToHex(tParam, &SymPackedSECU.data[tCountParam]);

  tParam = airflow;
  tCountParam += 4;//+ knock_r
  CharToHex(tParam, &SymPackedSECU.data[tCountParam]);

  tParam = packByte;
  tCountParam += 2;
  CharToHex(tParam, &SymPackedSECU.data[tCountParam]);

  tCountParam += 2;  // indx =32
  SymPackedSECU.data[tCountParam]=LAST_SYMBOL_BLOCK;
  SymPackedSECU.data[33]='$';

  return ((char*)&SymPackedSECU);
}


//-------------------------------------------------------------------

//----------------- AVR  -----------------------------
unsigned int GetBaudRate(int aAvrUBRR)
{
  unsigned int tReturnValue;
  switch(aAvrUBRR)
  {
    case  103:
        tReturnValue = 9600;
        break;
    case  68:
        tReturnValue = 14400;
        break;
    case  51:
        tReturnValue = 19200;
        break;
    case  34:
        tReturnValue = 28800;
        break;
    case  25:
        tReturnValue = 38400;
        break;
    case  16:
        tReturnValue = 57600;
        break;

    default:
     tReturnValue = 9600;
  };

  return tReturnValue;
}

unsigned int GetAvrUBRR(int aBaudRate)
{
  unsigned int tReturnValue;
  switch(aBaudRate)
  {
    case  9600:
        tReturnValue = 103;
        break;
    case  14400:
        tReturnValue = 68;
        break;
    case  19200:
        tReturnValue = 51;
        break;
    case  28800:
        tReturnValue = 34;
        break;
    case  38400:
        tReturnValue = 25;
        break;
    case  57600:
        tReturnValue = 16;
        break;

    default:
     tReturnValue = 103;
  };

  return tReturnValue;
}

//---------------------------------------------------------------------------


__fastcall TConsolLoggerForm::TConsolLoggerForm(TComponent* Owner)
        : TForm(Owner)
{

//---------------------TEST
//char tTest[]= {"00:00:08.04, 4514, 30.60, 59.85,11.16,-28.00,3.66, 5.90, 6,1,1,1,1$"};
//unsigned char*  tPtr = &tTest[67-1];

//---------------------

 FHandleLogSecu = -1;
 FHandleTableSecu = -1;

 FNoAccessColor = clBtnFace;
 FNoEditColor = clInfoBk;

 FCount_TxD = 0;
 FCount_RxD = 0;
 FCountDataReceived =0;

 FCountSymPacked = 0;
 FCountWTablePacked = 0;
 FContextReadCounter = 0;

 //FCountSymKB = 0;

 //    Tag
 //TxDShape->Tag  = (int)TxDShape->Brush->Color;
 RxDShape->Tag  = (int)RxDShape->Brush->Color;

 CE_OVERRUN_Shape->Tag =
           (int)CE_OVERRUN_Shape->Brush->Color;
 CE_TXFULL_Shape->Tag =
           (int)CE_TXFULL_Shape->Brush->Color;
 CE_RXOVER_Shape->Tag =
           (int)CE_RXOVER_Shape->Brush->Color;


 CountPackedLabeledEdit->EditLabel->Caption = "";
 CommPort->ComNumber = ComCSpinEdit->Value;

 __try
 {
   FRxDTimer = new TTimer(NULL);
   FRxDTimer->Enabled = false;
   FRxDTimer->Interval = 500;
   FRxDTimer->OnTimer = RxDTimerTimer;
 }
 catch(...)
 {
   FRxDTimer = NULL;
 }



 //BaudComboBox->Items->Add("110");
 //BaudComboBox->Items->Add("300");

 /*
 CommPort->Baud = 600;
 if(CommPort->Baud == 600)
  BaudComboBox->Items->Add("600");

 CommPort->Baud = 1200;
 if(CommPort->Baud == 1200)
     BaudComboBox->Items->Add("1200");

 CommPort->Baud = 2400;
 if(CommPort->Baud == 2400)
   BaudComboBox->Items->Add("2400");

 CommPort->Baud = 4800;
 if(CommPort->Baud == 4800)
     BaudComboBox->Items->Add("4800");
 */

 CommPort->Baud = 9600;
 if(CommPort->Baud == 9600)
 {
    BaudComboBox->Items->Add("9600");
    BaudConsolComboBox->Items->Add("9600");
 }

 CommPort->Baud = 14400;
 if(CommPort->Baud == 14400)
 {
    BaudComboBox->Items->Add("14400");
    BaudConsolComboBox->Items->Add("14400");
 }

 CommPort->Baud = 19200;
 if(CommPort->Baud == 19200)
 {
    BaudComboBox->Items->Add("19200");
    BaudConsolComboBox->Items->Add("19200");
 }

 CommPort->Baud = 28800;
 if(CommPort->Baud == 28800)
 {
   BaudComboBox->Items->Add("28800");
   BaudConsolComboBox->Items->Add("28800");
 }

 CommPort->Baud = 38400;
 if(CommPort->Baud == 38400)
 {
    BaudComboBox->Items->Add("38400");
    BaudConsolComboBox->Items->Add("38400");
 }


 CommPort->Baud = 57600;
 if(CommPort->Baud == 57600)
 {
   BaudComboBox->Items->Add("57600");
   BaudConsolComboBox->Items->Add("57600");
 }

 //BaudComboBox->Items->Add("115200");
 //BaudComboBox->Items->Add("128000");
 //BaudComboBox->Items->Add("256000");
 //BaudComboBox->Items->Add("230400");
 //BaudComboBox->Items->Add("460800");
 //BaudComboBox->Items->Add("921600");
 //BaudComboBox->Items->Add("1843200");
 //BaudComboBox->Text = "115200";
 BaudComboBox->ItemIndex = 0;
 BaudConsolComboBox->ItemIndex = 0;


 BaudComboBoxChange(NULL);
 //CheckIndicationGroup(CommPort->Open);

 DateTimeMaskEdit->Text = DateToStr(Date());
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::FormDestroy(TObject *Sender)
{
 if(FRxDTimer)
 {
   FRxDTimer->Enabled = false;
   delete FRxDTimer;
   FRxDTimer = NULL;
 }

 CommPort->Open = false;
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::ExitButtonClick(TObject *Sender)
{
   Close();
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::OpenPortBitBtnClick(TObject *Sender)
{

  // / 
  CommPort->Open = !CommPort->Open;
  CheckIndicationGroup(CommPort->Open);
  if(CommPort->Open)
  {
    CommPort->RTS=true;
    CommPort->DTR=false;

    FCount_TxD = 0;
    FCount_RxD = 0;
    FCountDataReceived =0;
    FCountSymPacked = 0;
    FCountWTablePacked = 0;
    FContextReadCounter =0;

    CountOutputLabel->Caption =0;
    CountInputLabel->Caption =0;

//-----------TEST
//    DebugMemo->Clear();


  }
  else if(SymSecuStartButton->Tag)//  
              SymSecuStartButtonClick(NULL);

}
//---------------------------------------------------------------------------
unsigned int MakeEndStr(char* aBuf)
{
    unsigned int tCount = 0;
    do
    {
      if(aBuf[tCount]==' ')
      {
        aBuf[tCount]=LAST_SYMBOL_BLOCK;
        tCount++;
        return tCount;
      }
    }while(++tCount < 100);

    return 0;
}

void __fastcall TConsolLoggerForm::CheckBufRxD(void)
{


   //  -   ?
 unsigned int tCountRxD = CommPort->InBuffUsed;

 if(tCountRxD)
 {
   if(FHandleLogSecu < 0 && FHandleTableSecu < 0)
   {
     FillMemory(
        ((void*)&FCurrentCommand.packed),
                 sizeof(LOGGER_PACKED),0xFF);

     if(tCountRxD > sizeof(LOGGER_PACKED))
             tCountRxD = sizeof(LOGGER_PACKED);

     CommPort->GetBlock(
         ((char*)&FCurrentCommand.packed),tCountRxD);

     FCurrentCommand.complete = TRUE;

     if(FCurrentCommand.packed.hostRequest==
                                   LOGGER_ANSWER)
     {
          //    tCountRxD
          tCountRxD--;

          //  
          for(unsigned int tCount=0;
            tCount < sizeof(LOGGER_PACKED); tCount++)
          {
            char tChar =
              ((char*)&FCurrentCommand.packed)[tCount];
            if(tChar == LAST_SYMBOL_BLOCK &&
                            tCount == tCountRxD)
            {
              //   -  OK!  
              //      
              switch(FCurrentCommand.packed.commandID)
              {
                case GET_CONTEXT :
                {

                  TDateTime tDateTime =Time();
                  tDateTime -= FTimeContextSave;
                  ContextCalcTimeLabel->Caption = tDateTime.TimeString();

                  //   SIZE_LOGGER_ID  +4 +6 +8 +10  +14 6x2
                  //     0 1 2 3 4 $$$ 30 32 34 36   40     50
                  // !,#,x,x,x,x, 22ID,NN,UU,fl,Date,unused,\r
                  // FileSize
                  int tValue =
                    StrToInt(&FCurrentCommand.packed.data[0],2);
                  FileSizeLabel->Caption=tValue;

                  BytePerSectorLabel->Caption="512";

                  //  
                  tValue =
                    StrToInt(&FCurrentCommand.packed.data[SIZE_LOGGER_ID+4],1);
                  ContextLoggerFilesCountEdit->Text = tValue;

                  //  
                  int tAvrDiv =
                    StrToInt(&FCurrentCommand.packed.data[SIZE_LOGGER_ID+6],1);
                  tValue = GetBaudRate(tAvrDiv);
                  ContextLoggerBaudEdit->Text = tValue;

                  //Flag
                  tValue =
                    StrToInt(&FCurrentCommand.packed.data[SIZE_LOGGER_ID+8],1);

                  ContextRequestTablesCheckBox->Checked =
                                            (tValue & NO_SAVE_CONTEXT_SECU);
                  ContextSmartRequestTablesCheckBox->Checked =
                                         (tValue & SMART_SAVE_CONTEXT_SECU);
                  ContextExportTablesCheckBox->Checked =
                                        (tValue & NO_WRITE_CONTEXT_TO_SECU);
                  ContextSmartLogCheckBox->Checked =
                                     (tValue & NO_LOGGED_NOT_WORKING_ENGINE);
                  ContextBinLogCheckBox->Checked = (tValue & BINARY_LOG_FORMAT);

                  //DateTime
                  tValue =
                    StrToInt(&FCurrentCommand.packed.data[SIZE_LOGGER_ID+10],2);
                  if(tValue==0xFFFF)
                      tValue= (((unsigned int)1<<5) + 1);

                  // ff_fdate: (struct ffblk Win32 dos.h)
                  // Bits 0-4  Day
                  // Bits 5-8  Month
                  // Bits 9-15 Years since 1980 (for example 9 here means 1989)
                  //(((unsigned int)(aYears-1980)<<9) +
                  //                 ((unsigned int)aMonth<<5) + aDay);
                  unsigned short tDay = tValue & 0x1F;
                  unsigned short tMonth = (tValue>>5) & 0xF;
                  unsigned short tYear = (tValue>>9) + 1980;
                  TDateTime tContextDate(tYear,tMonth,tDay);
                  ContextDateTimeLabel->Caption = tContextDate.DateString();

                  //   SD  
                  tValue =
                    StrToInt(&FCurrentCommand.packed.data[SIZE_LOGGER_ID+14],2);
                  SizeDiscLabel->Caption=tValue;

                  //     SD   unused[0]
                  tValue =
                    StrToInt(&FCurrentCommand.packed.data[SIZE_LOGGER_ID+18],1);
                  TimeWritePackedLabel->Caption = tValue*10;

                  //unused[3] Handle
                  tValue =
                    StrToInt(&FCurrentCommand.packed.data[SIZE_LOGGER_ID+20],1);
                  HandleLabel->Caption = (signed char)tValue;

                  //unused[4,5] Time
                  tValue =
                    StrToInt(&FCurrentCommand.packed.data[SIZE_LOGGER_ID+22],2);
                  int tSecond = (tValue & 0x1F)*2;
                  int tMinute = (tValue >>5) & 0x3F;
                  int tHour = (tValue >>11);

                  if(tAvrDiv==0xFF)
                         tAvrDiv= 103;
                  int tMillisecond =
                       1+tCountRxD*((tAvrDiv + 1))/1000;

                  TDateTime tTime =
                    EncodeTime(tHour, tMinute, tSecond, tMillisecond);

                  ContextTimeLabel->Caption=tTime.TimeString();

                  // ID
                  if(((unsigned char*)&FCurrentCommand.packed.data)[4]==0xFF)
                  {
                    ContextLoggerIdEdit->Text ="Empty string";
                  }
                  else
                  {
                    FCurrentCommand.packed.data[SIZE_LOGGER_ID + 4]=0;
                    ContextLoggerIdEdit->Text =
                      AnsiString(&((char*)(&FCurrentCommand.packed.data))[4]);
                  }

                  CheckAccessContextBox(true);

                  break;
                }
                case CHANGEMODE:
                {
                  /*
                  #define LEN_TEMPER_PAR  14
                  #define LEN_CARBUR_PAR  26
                  #define LEN_IDLREG_PAR  28
                  #define LEN_ANGLES_PAR  24
                  #define LEN_FUNSET_PAR  23
                  #define LEN_STARTR_PAR  11
                  #define LEN_CKPS_PAR    11
                  #define LEN_KNOCK_PAR   34
                  #define LEN_MISCEL_PAR  18
                  #define LEN_EDITAB_PAR   1528
                  */

                  //-------- !!!!! -----------
                  //    SECU
                  //   .

                  if(SymContextSecuButton->Tag)
                  {
                    if(FCurrentCommand.packed.data[0] == TEMPER_PAR)
                    {
                      //   SECU_SENSOR_DAT
                      CommPort->FlushOutBuffer();

                      Byte tOutBuf[]={"@j10101880180 "};
                      unsigned int tSize = MakeEndStr(tOutBuf);
                      if(tSize)
                      {
                          CommPort->PutBlock(tOutBuf,tSize);
                          CommPort->PutBlock(tOutBuf,tSize);
                      }
                      //StatusBar->Panels->Items[1]->Text="TEMPER_PAR";
                    }
                    else if(FCurrentCommand.packed.data[0] == CARBUR_PAR)
                    {
                      Byte tOutBuf[]={"@k05140578001880514057832 "};
                      unsigned int tSize = MakeEndStr(tOutBuf);
                      if(tSize)
                      {
                          CommPort->PutBlock(tOutBuf,tSize);
                          CommPort->PutBlock(tOutBuf,tSize);
                      }
                      //StatusBar->Panels->Items[1]->Text="CARBUR_PAR";
                    }
                    else if(FCurrentCommand.packed.data[0] == IDLREG_PAR)
                    {
                      Byte tOutBuf[]={"@l100040004000A038EFEC00140 "};
                      unsigned int tSize = MakeEndStr(tOutBuf);
                      if(tSize)
                      {
                          CommPort->PutBlock(tOutBuf,tSize);
                          CommPort->PutBlock(tOutBuf,tSize);
                      }
                      //StatusBar->Panels->Items[1]->Text="IDLREG_PAR";
                    }
                    else if(FCurrentCommand.packed.data[0] == ANGLES_PAR)
                    {
                      Byte tOutBuf[]={"@m06E0FEC00080006000600 "};
                      unsigned int tSize = MakeEndStr(tOutBuf);
                      if(tSize)
                      {
                          CommPort->PutBlock(tOutBuf,tSize);
                          CommPort->PutBlock(tOutBuf,tSize);
                      }
                      //StatusBar->Panels->Items[1]->Text="ANGLES_PAR";
                    }
                    else if(FCurrentCommand.packed.data[0] == FUNSET_PAR)
                    {
                      Byte tOutBuf[]={"@n00010840188000D20185 "};
                      unsigned int tSize = MakeEndStr(tOutBuf);
                      if(tSize)
                      {
                          CommPort->PutBlock(tOutBuf,tSize);
                          CommPort->PutBlock(tOutBuf,tSize);
                      }
                      //StatusBar->Panels->Items[1]->Text="FUNSET_PAR";
                    }
                    else if(FCurrentCommand.packed.data[0] == STARTR_PAR)
                    {
                      Byte tOutBuf[]={"@o0258028A "};
                      unsigned int tSize = MakeEndStr(tOutBuf);
                      if(tSize)
                      {
                          CommPort->PutBlock(tOutBuf,tSize);
                          CommPort->PutBlock(tOutBuf,tSize);
                      }
                      //StatusBar->Panels->Items[1]->Text="STARTR_PAR";
                    }
                    //else if(FCurrentCommand.packed.data[0] == FNNAME_DAT)
                    //else if(FCurrentCommand.packed.data[0] == ADCCOR_PAR)
                    //else if(FCurrentCommand.packed.data[0] == ADCRAW_DAT)
                    else if(FCurrentCommand.packed.data[0] == CKPS_PAR)
                    {
                      Byte tOutBuf[]={"@t11428040 "};
                      unsigned int tSize = MakeEndStr(tOutBuf);
                      if(tSize)
                      {
                          CommPort->PutBlock(tOutBuf,tSize);
                          CommPort->PutBlock(tOutBuf,tSize);
                      }
                      //StatusBar->Panels->Items[1]->Text="CKPS_PAR";
                    }
                    //else if(FCurrentCommand.packed.data[0] == OP_COMP_NC)
                    //else if(FCurrentCommand.packed.data[0] == CE_ERR_CODES)
                    else if(FCurrentCommand.packed.data[0] == KNOCK_PAR)
                    {
                      Byte tOutBuf[]={"@w12A000003201700800008020003E802 "};
                      unsigned int tSize = MakeEndStr(tOutBuf);
                      if(tSize)
                      {
                          CommPort->PutBlock(tOutBuf,tSize);
                          CommPort->PutBlock(tOutBuf,tSize);
                      }
                      //StatusBar->Panels->Items[1]->Text="KNOCK_PAR";
                    }
                    //else if(FCurrentCommand.packed.data[0] == CE_SAVED_ERR)
                    //else if(FCurrentCommand.packed.data[0] == FWINFO_DAT)
                    else if(FCurrentCommand.packed.data[0] == MISCEL_PAR)
                    {
                      Byte tOutBuf[]={"@z00CF0811964000A "};
                      unsigned int tSize = MakeEndStr(tOutBuf);
                      if(tSize)
                      {
                          CommPort->PutBlock(tOutBuf,tSize);
                          CommPort->PutBlock(tOutBuf,tSize);
                      }
                      //StatusBar->Panels->Items[1]->Text="MISCEL_PAR";
                    }
                    else if(FCurrentCommand.packed.data[0] == EDITAB_PAR)
                    {
                      char* tOutBuf[40]={
                      "@{02801E1E2022262C38424647474747484949 ",
                      "@{02901E1E202226323C4547484848494A4B4B ",
                      "@{02A01E1E2022263440484A4A4A4A4A4C4C4C ",
                      "@{02B01E242830363C42484C4C4C4C4C4E4E4E ",
                      "@{02C01B242830363C42484C4C4C4C4C4E4E4E ",
                      "@{02D019242830363C42464A4A4A4A4A4B4B4B ",
                      "@{02E017242830363C42464A4A4A4A4A4A4A4A ",
                      "@{02F015242830363C42434343434445494949 ",
                      "@{0300221C1916130F0C0A0705020000FDF6EC ",
                      "@{0400TEST--TABLE-1                    ",
                      "@{100000000000000000000004080C10141414 ",
                      "@{11000A101922222222222428373737373737 ",
                      "@{12000C0D0F12171B1E22252A2C2E2E303030 ",
                      "@{12100D0E1013171B1F23262C2E2F30313233 ",
                      "@{12200D0E1013171920262A30343436363838 ",
                      "@{12300D0E1014171F24272E34383838383A3A ",
                      "@{12400D0E0E161820262A363A3C3C3C3B3C3C ",
                      "@{12500D0E10191F252E35393C4040403F4040 ",
                      "@{126018181A1F252B38404040444444434444 ",
                      "@{12701C1E222A313742454848484848474848 ",
                      "@{1280222528303840484A4A4A4A4A4A4A4A4A ",
                      "@{12902D30343B3F434C4C4C4C4C4C4C4C4D4D ",
                      "@{12A0363A3E4142454C4E4E4E4C4C4C4C4E50 ",
                      "@{12B0383C3F44454C4E4E4E4C4C4C4C4C4F51 ",
                      "@{12C0393C3F4244454C4C4C4C4C4C4C4C4F53 ",
                      "@{12D0383B3D4042444A4C4C4C4C4C4C4C4F55 ",
                      "@{12E033373B3E4042484A4A4A4A4A4A4B5158 ",
                      "@{12F03033363A3D4044484848494A4A4D525B ",
                      "@{1300221C1916130F0C0A0705020000FDF6EC ",
                      "@{1400TEST--TABLE-2                    ",
                      "@{000000000000000000000004080C10141414 ",
                      "@{01000A101922222222222428373737373737 ",
                      "@{02000C0C0E10111214161B1D1F2122242727 ",
                      "@{02100E0E1011121315171C20242526262929 ",
                      "@{022010101214151617191E22262729282C2C ",
                      "@{0230121214161717181D21262A2B2C2D2F2F ",
                      "@{024014141618191B232E3030303031333333 ",
                      "@{02501616181A1B232A323C3A3A3A3B383838 ",
                      "@{026018181A1E2326323A3F3F3F3F3F3F3F3F ",
                      "@{02701B1B2020242C36404343434343464646 "};


                      for(unsigned int tCount =0; tCount < 40; tCount++)
                      {
                        unsigned int tSize = MakeEndStr(tOutBuf[tCount]);
                        if(tSize)
                        {
                          CommPort->PutBlock(tOutBuf[tCount],tSize);
                        }
                        //StatusBar->Panels->Items[1]->Text=AnsiString("1) EDITAB_PAR ")+tCount;
                      }
                      for(unsigned int tCount =0; tCount < 20; tCount++)
                      {
                        unsigned int tSize = MakeEndStr(tOutBuf[tCount]);
                        if(tSize)
                        {
                          CommPort->PutBlock(tOutBuf[tCount],tSize);
                        }
                        //StatusBar->Panels->Items[1]->Text=AnsiString("2) EDITAB_PAR ")+tCount;
                      }
                      StatusBar->Panels->Items[1]->Text=
                        AnsiString("All adjustment SECU is read");
                      SymContextSecuButtonClick(SymContextSecuButton);
                    }
                    else if(FCurrentCommand.packed.data[0] == SENSOR_DAT)
                    {
                       AnsiString tStr = StatusBar->Panels->Items[1]->Text;
                       if(tStr.Pos("All") ==0)
                       {
                         //     
                         //   .  
                         //   
                         StatusBar->Panels->Items[1]->Text=
                          "Logger does not require adjustment. Ready for loggs";
                         SymContextSecuButtonClick(SymContextSecuButton);
                       }
                       else
                        StatusBar->Panels->Items[1]->Text=
                                                      "Ready for loggs";
                    }

                  }//if(SymContextSecuButton->Tag)
                  else if(FCurrentCommand.packed.data[0] == SENSOR_DAT)
                  {

                      StatusBar->Panels->Items[1]->Text="Ready for loggs";
                  }
                  else
                  {
                      StatusBar->Panels->Items[1]->Text=
                               "Console no ready for read adjustment SECU";
                  }

                  break;
                }//case CHANGEMODE:
                case DEGUG_STEP:
                {
                  break;
                }

                //default:
              };//switch
              break;
            }//if(tChar == LAST_SYMBOL_BLOCK...
          }//for(int tCount=0;...
     }//if(FCurrentCommand.packed.hostRequest...
     else
     {
         /*
         // Debug
         char* tPtr = (char*)&FCurrentCommand.packed;
         for(unsigned int tCount = 0;tCount < tCountRxD; tCount++)
         {
           if(tPtr[tCount]==0)
                   tPtr[tCount]='?';
         }
         tPtr[tCountRxD]=0;
         //DebugMemo->Lines->Add(AnsiString(tPtr));
         */

     }

     FCurrentCommand.size = 0;
     CommPort->FlushInBuffer();
   }//if(FHandleLogSecu <0)
   else if(FHandleTableSecu >=0)
   {

      //       SECU
      if(ReadWTableButton->Tag)
      {

        //   SECU
        char* tBuff  = new char[tCountRxD+1];
        CommPort->GetBlock(tBuff,tCountRxD);
        //     
        if(FCountWTablePacked==0)
        {
          for(unsigned int tCount =0 ; tCount < tCountRxD; tCount++)
          {
            if(tBuff[tCount]==EDITAB_PAR)
            {
              /*88888@{010000000000000000000004080C10141414\r*/
              tCount--;
              tCountRxD-=tCount;
              FileWrite(FHandleTableSecu, &tBuff[tCount],tCountRxD);
              delete []tBuff;

              FCountWTablePacked+=tCountRxD;
              break;

            }
          }
        }
        else //if(FCountWTablePacked)
        {
          #define SIZE_TABLE_RAM      (19*39+23)
          #define SIZE_ALL_TABLE_RAM  SIZE_TABLE_RAM*2
          unsigned int tNeedSize = SIZE_ALL_TABLE_RAM - FCountWTablePacked;
          if (tCountRxD > tNeedSize)
          {
            //    
            tCountRxD = tNeedSize;

          }

          FCountWTablePacked+=tCountRxD;

          FileWrite(FHandleTableSecu, tBuff,tCountRxD);
          delete []tBuff;

          if(FCountWTablePacked >= SIZE_ALL_TABLE_RAM)
          {

            FileClose(FHandleTableSecu);
            FHandleTableSecu=-1;
            ReadWTableButton->Tag=0;
            ReadWTableButton->Enabled = true;

            //    SENSOR_DAT :
            //    !hq\r
            char* tBuf = FPackedSymSecu.GetBuffPackedSECU();
            tBuf[0] = REQUEST_TO_SECU;
            tBuf[1] = CHANGEMODE;
            tBuf[2] = SECU_SENSOR_DAT;
            tBuf[3] = LAST_SYMBOL_BLOCK;

            CommPort->PutBlock(tBuf,4);

          }//if(FCountWTablePacked == 19)
        }//if(tBuff[1]==EDITAB_PAR)
      }//if(ReadWTableButton->Tag)
      else if(ReadParamButton->Tag)
      {
        //  ReadParamButton->Tag   
        unsigned int tSizeBuf = tCountRxD+1;
        char* tBuff  = new char[tSizeBuf];
        FillMemory(((void*)tBuff),sizeof(char)*tSizeBuf,0x20);

        CommPort->GetBlock(tBuff,tCountRxD);

        //     
        unsigned char tFound = false;
        unsigned int tCount =0;
        for(; tCount < tCountRxD; tCount++)
        {
            if(tBuff[tCount]==TEMPER_PAR && FContextReadCounter==0)
            {
              //----TEST
              tCount--;
              tCountRxD -= tCount;
              if(tCountRxD >= 14)
              {

                if(FTickCount)
                {
                  unsigned long tTime  = GetTickCount() - FTickCount;
                  //AnsiString tTimeStr(tTime);
                  StatusBar->Panels->Items[1]->Text =  tTime;
                  //-------
                }

                FContextReadCounter=1;
                tCountRxD = 14; //  
                //if(!FTickCount)
                 tFound = CARBUR_PAR; //    
                //else FTickCount =0; //  
              }
              break;
            }
            if(tBuff[tCount]==CARBUR_PAR  && FContextReadCounter ==1)
            {

              tCount--;
              tCountRxD -= tCount;
              if(tCountRxD >= 26)
              {
                 FContextReadCounter=2;
                 tCountRxD = 26; //  
                 tFound = IDLREG_PAR; //    
              }
              break;
            }
            if(tBuff[tCount]==IDLREG_PAR   && FContextReadCounter ==2)
            {

              tCount--;
              tCountRxD -= tCount;
              if(tCountRxD >= 28)
              {
                 FContextReadCounter=3;
                 tCountRxD = 28; //  
                 tFound = ANGLES_PAR; //    
              }
              break;
            }
            if(tBuff[tCount]==ANGLES_PAR   && FContextReadCounter ==3)
            {

              tCount--;
              tCountRxD -= tCount;
              if(tCountRxD >= 24)
              {
                 FContextReadCounter =4;
                 tCountRxD = 24; //  
                 tFound = FUNSET_PAR; //    
              }
              break;
            }
            if(tBuff[tCount]==FUNSET_PAR   && FContextReadCounter ==4)
            {

              tCount--;
              tCountRxD -= tCount;
              if(tCountRxD >= 23)
              {
                FContextReadCounter =5;
                tCountRxD = 23; //  
                tFound = STARTR_PAR; //    
              }
              break;
            }
            if(tBuff[tCount]==STARTR_PAR   && FContextReadCounter ==5)
            {

              tCount--;
              tCountRxD -= tCount;
              if(tCountRxD >= 11)
              {
                 FContextReadCounter =6;
                 tCountRxD = 11; //  
                 tFound = CKPS_PAR; //    
              }
              break;
            }
            if(tBuff[tCount]==CKPS_PAR   && FContextReadCounter ==6)
            {

              tCount--;
              tCountRxD -= tCount;
              if(tCountRxD >= 11)
              {
                FContextReadCounter =7;
                tCountRxD = 11; //  
                tFound = KNOCK_PAR; //    
              }
              break;
            }
            if(tBuff[tCount]==KNOCK_PAR   && FContextReadCounter ==7)
            {

              tCount--;
              tCountRxD -= tCount;
              if(tCountRxD >= 34)
              {
                FContextReadCounter=8;
                tCountRxD = 34; //  
                tFound = MISCEL_PAR; //    
              }
              break;
            }
            if(tBuff[tCount]==MISCEL_PAR   && FContextReadCounter ==8)
            {

              tCount--;
              tCountRxD -= tCount;
              if(tCountRxD >= 18)
              {
                FContextReadCounter = 9;
                tCountRxD = 18; //  
                //tFound = SENSOR_DAT; //    
                tFound = EDITAB_PAR;
                FCountWTablePacked=0;
              }
              break;
            }
            else if (tBuff[tCount]==EDITAB_PAR || FContextReadCounter > 9)
            {
               //   SECU
               FContextReadCounter++;

               if(tCount)
               {
                  tCount--;
                  tCountRxD -= tCount;
               }

               #define SIZE_TABLE_RAM      (19*39+23)
               #define SIZE_ALL_TABLE_RAM  SIZE_TABLE_RAM*2
               unsigned int tNeedSize = SIZE_ALL_TABLE_RAM - FCountWTablePacked;
               if (tCountRxD > tNeedSize)
               {
                 //    
                 tCountRxD = tNeedSize;

               }

               FCountWTablePacked+=tCountRxD;


               if(FCountWTablePacked >= SIZE_ALL_TABLE_RAM)
               {

                  //    SENSOR_DAT :
                  tFound = SECU_SENSOR_DAT;

               }//if(FCountWTablePacked == 19)
               else
                 tFound = 1;

               break;

            }//else if (tBuff[tCount]==EDITAB_PAR)
        }//for(; tCount < tCountRxD; tCount++)

        if(tFound)
        {
          FileWrite(FHandleTableSecu,
                         &tBuff[tCount],tCountRxD);
          if(tFound == SENSOR_DAT)
          {
            FileClose(FHandleTableSecu);
            FHandleTableSecu =-1;
            ReadParamButton->Tag = 0;
            ReadParamButton->Enabled = true;
          }

          if(tFound > 1)
          {
            char* tBufOut = FPackedSymSecu.GetBuffPackedSECU();
            tBufOut[0] = REQUEST_TO_SECU;
            tBufOut[1] = CHANGEMODE;
            tBufOut[2] = tFound;
            tBufOut[3] = LAST_SYMBOL_BLOCK;
            CommPort->PutBlock(tBufOut,4);
          }

        }//if(tFound)
        else if(!FContextReadCounter)
        {
          FileWrite(FHandleTableSecu,
                              tBuff,tCountRxD);
        }

        delete []tBuff;

      }

   }//else if(FHandleTableSecu >=0)
   else
   {
     //   SECU
     char* tBuff  = new char[tCountRxD+1];
     CommPort->GetBlock(tBuff,tCountRxD);

     FileWrite(FHandleLogSecu, tBuff,tCountRxD);
     delete []tBuff;
   }
 }//if(tCountRxD)
}

void __fastcall TConsolLoggerForm::RxDTimerTimer(TObject *Sender)
{
 if(FRxDTimer)
 {
   //   ,    
   //       
   //    ,  
   //   -  

   FRxDTimer->Enabled = false;
   CheckAccessContextBox(false);


   //   
   //TxDShape->Brush->Color = (TColor)TxDShape->Tag;
   RxDShape->Brush->Color = FNoEditColor;//(TColor)RxDShape->Tag;
   //DataReceivedShape->Brush->Color = DataReceivedShape->Tag;


   if(FRxDTimer)
       FRxDTimer->Tag=0;

 }//if(FRxDTimer)
}

//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::GetContextButtonClick(TObject *Sender)
{


  //   
  if(FRxDTimer && !FRxDTimer->Tag)
  {
     //      

     ContextLoggerIdEdit->Text="";
     ContextLoggerBaudEdit->Text="";
     ContextLoggerFilesCountEdit->Text="";
     SizeDiscLabel->Caption="";
     BytePerSectorLabel->Caption="";

     if(CommPort->Open)
     {
       //ZeroMemory(((void*)&aCommand),sizeof(COMMAND));

      FillMemory(
           ((void*)&FCurrentCommand.packed),
                      sizeof(LOGGER_PACKED),0xFF);

      FCurrentCommand.size = GET_CONTEXT_SIZE;
      FCurrentCommand.complete = FALSE;
      FCurrentCommand.packed.hostRequest = HOST_REQUEST;
      FCurrentCommand.packed.commandID = GET_CONTEXT;
      FCurrentCommand.packed.data[0] = LAST_SYMBOL_BLOCK;

      FRxDTimer->Enabled = true;
      FRxDTimer->Tag=1;

      CommPort->FlushInBuffer();
      CommPort->FlushOutBuffer();
      //   GET_CONTEXT
      int tSize = CommPort->PutBlock(
           ((char*)&FCurrentCommand.packed), GET_CONTEXT_SIZE);

     }//if(CommPort->Open)
  }//if(FRxDTimer && !FRxDTimer->Enabled)
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::SetContextButtonClick(TObject *Sender)
{
  //    
  if(FRxDTimer && !FRxDTimer->Tag)
  {
     //      
     if(CommPort->Open)
     {
       //ZeroMemory(((void*)&aCommand),sizeof(COMMAND));

      FillMemory(
           ((void*)&FCurrentCommand.packed),
                      sizeof(LOGGER_PACKED),0x20);// 


      //------------- Header packed ---------------

      //FCurrentCommand.size = SET_CONTEXT_SIZE;
      FCurrentCommand.complete = FALSE;
      FCurrentCommand.packed.hostRequest = HOST_REQUEST;
      FCurrentCommand.packed.commandID = SET_CONTEXT;

      //------------ Reserved[2] ------------------
      int tCount =0;
      FCurrentCommand.packed.data[tCount++]='1';
      FCurrentCommand.packed.data[tCount++]='2';
      FCurrentCommand.packed.data[tCount++]='3';
      FCurrentCommand.packed.data[tCount++]='4';

      // StringID
      strncpy(
       &FCurrentCommand.packed.data[tCount],
                LoggerIdEdit->Text.c_str(),SIZE_LOGGER_ID);


      //for(;tCount < SIZE_LOGGER_ID+4; tCount++)
      {

      //   FCurrentCommand.packed.data[tCount]=LoggerIdEdit->Text.c_str();
      }

      //-------------- FilesCount --------------
      tCount+=SIZE_LOGGER_ID;

      Byte tCharValue = (Byte)LoggerFilesCountEdit->Text.ToInt();
      CharToHex(tCharValue, &FCurrentCommand.packed.data[tCount]);

      //-------------- BaudRate ----------------

      tCount+=2;
      tCharValue = GetAvrUBRR(BaudConsolComboBox->Text.ToInt());
      CharToHex(tCharValue, &FCurrentCommand.packed.data[tCount]);

      //-------------- Flags --------------------
      tCount+=2;
      tCharValue = 0;
      if(RequestTablesCheckBox->Checked)
              tCharValue |= NO_SAVE_CONTEXT_SECU;
      if(SmartRequestTablesCheckBox->Checked)
              tCharValue |= SMART_SAVE_CONTEXT_SECU;
      if(ExportTablesCheckBox->Checked)
              tCharValue |= NO_WRITE_CONTEXT_TO_SECU;
      if(SmartLogCheckBox->Checked)
              tCharValue |= NO_LOGGED_NOT_WORKING_ENGINE;
      if(BinLogCheckBox->Checked)
              tCharValue |= BINARY_LOG_FORMAT;

      CharToHex(tCharValue,
                 &FCurrentCommand.packed.data[tCount]);
      //FCurrentCommand.packed.data[tCount++] = '0';
      //FCurrentCommand.packed.data[tCount++] = '0';

      // Date
      tCount+=2;
      TDateTime tDateTime(DateTimeMaskEdit->Text);
      unsigned short tYear;
      unsigned short tMonth;
      unsigned short tDay;

      tDateTime.DecodeDate(&tYear,&tMonth,&tDay);
      unsigned int tFileDate =
        (((tYear-1980)<<9) + (tMonth<<5) + tDay);
      IntToHex(tFileDate, &FCurrentCommand.packed.data[tCount]);


      //------------------ unused[6] ------------

      //----- Time -----
      // ff_ftime: (struct ffblk Win32 dos.h)
      // Bits 0 to  4   The result of seconds divided by 2
      //               (for example 10 here means 20 seconds)
      //  Bits 5 to  10  Minutes
      //  Bits 11 to 15  Hours

      Word tHour, tMin, tSec, tMSec;
      tDateTime = Time();
      tDateTime.DecodeTime(&tHour, &tMin, &tSec, &tMSec);
      Word tTime = ( (tHour<<11) + (tMin<<5)+ tSec/2);

      FTimeContextSave = tDateTime;

      // unused[0,1]
      tCount+=4;
      IntToHex(tTime, &FCurrentCommand.packed.data[tCount]);
      tCount+=4;

      //unused[2]
      FCurrentCommand.packed.data[tCount++] = '0';
      FCurrentCommand.packed.data[tCount++] = '0';
      //unused[3]
      FCurrentCommand.packed.data[tCount++] = '0';
      FCurrentCommand.packed.data[tCount++] = '0';
      //unused[4]
      FCurrentCommand.packed.data[tCount++] = '0';
      FCurrentCommand.packed.data[tCount++] = '0';
      //unused[5]
      FCurrentCommand.packed.data[tCount++] = '0';
      FCurrentCommand.packed.data[tCount++] = '0';



      FCurrentCommand.packed.data[tCount] = LAST_SYMBOL_BLOCK;
      tCount += (2+1); //HOST_REQUEST+SET_CONTEXT+1  ~55

      FCurrentCommand.size = tCount;

      FRxDTimer->Enabled = true;
      FRxDTimer->Tag=1;

      CommPort->FlushInBuffer();
      CommPort->FlushOutBuffer();
      //   GET_SYSTEM_STATUS
      int tSize = CommPort->PutBlock(
           ((char*)&FCurrentCommand.packed), tCount);

     }//if(CommPort->Open)
  }//if(FRxDTimer && !FRxDTimer->Enabled)
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::SymSecuStartButtonClick(TObject *Sender)
{
  if(SymSecuStartButton->Tag ==1)
  {
    //   - 
    SymSecuStartButton->Caption = "Start";
    SymSecuStartButton->Tag =0;
    //SendSymPackedTimer->Interval = 0;
    SendSymPackedTimer->Enabled = false;

  }
  else
  {
    //   - 
    FAutoGetContext = SYM_REFRESH_STATUS_PERIOD;
    SymSecuStartButton->Caption = "Stop";
    SymSecuStartButton->Tag =1;
    SendSymPackedTimer->Interval =
                           PeriodSymCSpinEdit->Value;
    FPackedSymSecu.Initialize(SymShiftRadioButton->Checked);

    CommPort->PutBlock(FPackedSymSecu.GetBuffPackedSECU(),
                                      SIZE_SECU_SENSOR_DAT);
    SendSymPackedTimer->Enabled = true;

    FCountSymPacked=1;
    CountPackedLabeledEdit->Text = FCountSymPacked;
    //    35 
    //       67 
    SizeEdit->Text = ((double)FCountSymPacked)*SYM_LOG_LINE_LEN/1024.0;
  }
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::SendSymPackedTimerTimer(TObject *Sender)
{
  //    SECU
  if(!FAutoGetContext)
  {
      //  
      GetContextButtonClick(NULL);
      FAutoGetContext=SYM_REFRESH_STATUS_PERIOD;

  }
  else
  {
    FAutoGetContext--;

    SendSymPackedTimer->Interval = PeriodSymCSpinEdit->Value;
    //FPackedSymSecu.MakeSymPackedSECU();
    CommPort->PutBlock(FPackedSymSecu.MakeSymPackedSECU(),
                                          SIZE_SECU_SENSOR_DAT);
    FCountSymPacked++;
    CountPackedLabeledEdit->Text = FCountSymPacked;

    //    55 
    //       67 
    SizeEdit->Text = ((double)FCountSymPacked)*SYM_LOG_LINE_LEN/1024.0;
  }
}
//---------------------------------------------------------------------------


//------------------    -------------------------------

//--------------      --------------------

void TConsolLoggerForm::CheckIndicationGroup(bool aOpenPort)
{
  CheckAccessContextBox(aOpenPort);
  CheckAccessSymBox(aOpenPort);
  CheckAccessConsolBox(aOpenPort);
  CheckAccessRequestButtons(aOpenPort);

  OpenPortBitBtn->Tag = aOpenPort;
  if(aOpenPort)
     OpenPortBitBtn->Caption = " ";
  else
     OpenPortBitBtn->Caption = " ";
}
//---------------------------------------------------------------------------

void TConsolLoggerForm::CheckAccessSymBox(bool aOpenPort)
{
  //     
  DeviceGroupBox->Enabled = aOpenPort;
  SymSecuStartButton->Enabled = aOpenPort;
  SymParametrGroupBox->Enabled = aOpenPort;
  PeriodSymCSpinEdit->Enabled = aOpenPort;

  TColor tColor = (aOpenPort) ? FNoEditColor : FNoAccessColor;
  PeriodSymCSpinEdit->Color = tColor;
  CountPackedLabeledEdit->Color = tColor;
  SizeEdit->Color = tColor;
}

void TConsolLoggerForm::CheckAccessContextBox(bool aOpenPort)
{
  //     
  DeviceGroupBox->Enabled = aOpenPort;
  //GetContextButton->Enabled  = aOpenPort;
  //SetContextButton->Enabled  = aOpenPort;
  //GetDiscInfoButton->Enabled = aOpenPort;
  TColor tColor = (aOpenPort) ? FNoEditColor : FNoAccessColor;

  ContextLoggerBaudEdit->Color = tColor;
  ContextLoggerFilesCountEdit->Color = tColor;
  ContextLoggerIdEdit->Color = tColor;
}

//---------------------------------------------------------------------------

void TConsolLoggerForm::CheckAccessConsolBox(bool aOpenPort)
{

  GetContextButton->Enabled = aOpenPort;
  SetContextButton->Enabled = aOpenPort;
  InterceptSECUButton->Enabled = aOpenPort;
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::BaudComboBoxChange(TObject *Sender)
{
  //------   OnChange BaudComboBox------

  CommPort->Baud = StrToInt(BaudComboBox->Text);
  CheckIndicationGroup(CommPort->Open);

  //    
  int tMinPeriod_msec =
    (10000.0/CommPort->Baud)*(SIZE_SECU_SENSOR_DAT);

  int tTruncation = SYM_LOG_MAX_PERIOD;
  for(;tTruncation > tMinPeriod_msec;
                     tTruncation-=SYM_LOG_STEP_PERIOD);

  tTruncation +=SYM_LOG_STEP_PERIOD;
  if(tMinPeriod_msec > SYM_LOG_MIN_PERIOD)
  {
       PeriodSymCSpinEdit->MinValue = tTruncation;
  }
  else
    PeriodSymCSpinEdit->MinValue = SYM_LOG_MIN_PERIOD;

  if(PeriodSymCSpinEdit->Value < PeriodSymCSpinEdit->MinValue)
         PeriodSymCSpinEdit->Value = PeriodSymCSpinEdit->MinValue;
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::ComCSpinEditChange(TObject *Sender)
{
  //------   OnChange ComCSpinEdit------

  CommPort->Open = false;
  CommPort->ComNumber = ComCSpinEdit->Value;
  CheckIndicationGroup(CommPort->Open);
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::TimerStatusInfoTimer(TObject *Sender)
{
 //------   OnTimer StatusInfoTimer ------
 //     StatusBar 800 ms
 if(CommPort->Open != StatusBar->Tag)
 {
   AnsiString tStatusPort;
   if(CommPort->Open)
   {
     StatusBar->Tag=1;
     tStatusPort = "  "+
            IntToStr(CommPort->Baud)+" /";
   }
   else
   {
     tStatusPort = "  ";
     StatusBar->Tag=0;
   }
   StatusBar->Panels->Items[0]->Text = AnsiString("Port  ")+
        AnsiString(CommPort->ComNumber)+tStatusPort;
 }//if(CommPort->Open != StatusBar->Tag)

 if(TxDShape->Tag)
 {
   if(TxDShape->Brush->Color == FNoEditColor)
              TxDShape->Brush->Color = clBlue;
   else
     TxDShape->Brush->Color = FNoEditColor;
 }
 else
  TxDShape->Brush->Color = FNoEditColor;//((TColor)TxDShape->Tag);

 TxDShape->Tag=0;

 //RxDShape->Brush->Color = ((TColor)RxDShape->Tag);

}
//---------------------------------------------------------------------------

//---------- ,    . ------------------------------

void __fastcall TConsolLoggerForm::CommPortTxDSignal(TObject *Sender)
{
  //------   OnTxDSignal CommPort ------
  //   EV_TXEMPTY 
  //      
  TxDShape->Tag=1;

  //if(TxDShape->Brush->Color == ((TColor)TxDShape->Tag))
  //   TxDShape->Brush->Color = clBlue;
  //else
  // TxDShape->Brush->Color = ((TColor)TxDShape->Tag);

  FCount_TxD++;
  CountOutputLabel->Caption = FCount_TxD;
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::CommPortRxDSignal(TObject *Sender)
{
  //------   OnRxDSignal CommPort ------
  //   EV_RXCHAR 
  //       
  RxDShape->Brush->Color = clLime;
  FCount_RxD++;
  CountInputLabel->Caption = FCount_RxD;
  CheckBufRxD();
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::CommPortRxEventSignal(TObject *Sender)
{
  //------   OnRxEventSignal CommPort ------
  //   EV_RXFLAG 
  //     EvtChar  DCB
  //      
  // ----     --------

  //FCount_RxD++;
  //TxDShape->Brush->Color = clLime;
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::CommPortDataReceived(TObject *Sender, DWORD Count)
{
  //------   OnDataReceived CommPort ------
  //  
  // WaitForSingleObject(FComPort->reEvent, INFINITE) == WAIT_OBJECT_0
  // ????
  //       !!! -- ????

  FCountDataReceived++;
  //DataReceivedLabel->Caption = FCountDataReceived;
  //DataReceivedShape->Brush->Color = clLime;

  /*
  if(FRxDTimer)
    FRxDTimer->Enabled = false;

  RxDShape->Brush->Color = clLime;

  if(FRxDTimer)
    FRxDTimer->Enabled = true;
  */
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::SymStatRadioButtonClick(TObject *Sender)
{
  //     
  SymShiftRadioButton->Checked = false;
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::SymShiftRadioButtonClick(TObject *Sender)
{
  //     
  SymStatRadioButton->Checked = false;
}
//---------------------------------------------------------------------------


//------------------------ TEST ---------------------------------------------
void __fastcall TConsolLoggerForm::ClockTimerTimer(TObject *Sender)
{
  ClockEdit->Text = TimeToStr(Time());
}
//---------------------------------------------------------------------------


//---------------    ----------------------
void __fastcall TConsolLoggerForm::CommPortErrorSignal(
                                 TObject *Sender,int aError)
{
  if(aError & CE_OVERRUN)
           CE_OVERRUN_Shape->Brush->Color=clRed;//clBlue;
  else
   CE_OVERRUN_Shape->Brush->Color = (TColor)CE_OVERRUN_Shape->Tag;

  if(aError & CE_TXFULL)
           CE_TXFULL_Shape->Brush->Color=clRed;
  else
   CE_TXFULL_Shape->Brush->Color = (TColor)CE_TXFULL_Shape->Tag;

  if(aError & CE_RXOVER)
           CE_RXOVER_Shape->Brush->Color=clRed;
  else
   CE_RXOVER_Shape->Brush->Color = (TColor)CE_RXOVER_Shape->Tag;

}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::InterceptSECUButtonClick(TObject *Sender)
{
  // ------   SECU   -----------
  if(InterceptSECUButton->Tag ==1)
  {
    //   - 
    InterceptSECUButton->Caption = "  SECU  ";
    InterceptSECUButton->Tag =0;
    if(FHandleLogSecu > 0)
    {
       char tBuf[]={"    END SECTION    "};
       tBuf[0]=0x0D;
       tBuf[1]=0x0A;
       tBuf[17]=0x0D;
       tBuf[18]=0x0A;
        FileWrite(FHandleLogSecu,tBuf,19);
        FileClose(FHandleLogSecu);
        FHandleLogSecu=-1;
    }
  }
  else
  {
    //   - 

    InterceptSECUButton->Caption = "";
    InterceptSECUButton->Tag =1;

    AnsiString tFileName(GetCurrentDir()+"\\LogSECU.shx" );
    if(FileExists(tFileName))
    {
      FHandleLogSecu = FileOpen(tFileName, fmOpenWrite);
      if(FHandleLogSecu >0)
      {
        int tFileLength = FileSeek(FHandleLogSecu,0,2);

      }
    }
    else
      FHandleLogSecu = FileCreate(tFileName);
  }

}
//---------------------------------------------------------------------------

void TConsolLoggerForm::CheckAccessRequestButtons(bool aOpenPort)
{
  ReadWTableButton->Enabled = aOpenPort;
  WriteWTableButton->Enabled = aOpenPort;
  ReadParamButton->Enabled = aOpenPort;
  SymContextSecuButton->Enabled = aOpenPort;
}

void __fastcall TConsolLoggerForm::ReadWTableButtonClick(TObject *Sender)
{
  //------      SECU ------
  /*
      : !h{\r
      38  :
     @{xxxx00000000000000000004080C10141414\r
        SENSOR_DAT :
     !hq\r
  */

   //FPackedSymSecu.Initialize(false);

   AnsiString tFileName(GetCurrentDir()+"\\WTableSECU.tbl");
   if(FileExists(tFileName))
   {
     DeleteFile(tFileName);
   }
   FHandleTableSecu = FileCreate(tFileName);

   ReadWTableButton->Tag=1; //   
   ReadWTableButton->Enabled = false;
   FCountWTablePacked =0;

   char* tBuf = FPackedSymSecu.GetBuffPackedSECU();
   tBuf[0] = REQUEST_TO_SECU;
   tBuf[1] = CHANGEMODE;
   tBuf[2] = EDITAB_PAR;
   tBuf[3] = LAST_SYMBOL_BLOCK;

   CommPort->PutBlock(tBuf,4);
}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::WriteWTableButtonClick(TObject *Sender)
{
  //      SECU


}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::ReadParamButtonClick(TObject *Sender)
{
//------------ TEST    SECU
   AnsiString tFileName(GetCurrentDir()+"\\parametrSECU.prm");
   if(FileExists(tFileName))
   {
     DeleteFile(tFileName);
   }
   FHandleTableSecu = FileCreate(tFileName);

   ReadParamButton->Tag=1; //   
   ReadParamButton->Enabled = false;
   FCountWTablePacked =0;
   FContextReadCounter = 0;

   char* tBuf = FPackedSymSecu.GetBuffPackedSECU();
   tBuf[0] = REQUEST_TO_SECU;
   tBuf[1] = CHANGEMODE;
   tBuf[2] = TEMPER_PAR;
   tBuf[3] = LAST_SYMBOL_BLOCK;

   CommPort->PutBlock(tBuf,4);
   FTickCount  = GetTickCount();

}
//---------------------------------------------------------------------------

void __fastcall TConsolLoggerForm::SymContextSecuButtonClick(TObject *Sender)
{
 if(Sender) //     !
 {
   //   SECU    
   bool tEnableSym = SymContextSecuButton->Tag;
   ReadWTableButton->Enabled = tEnableSym;
   WriteWTableButton->Enabled = tEnableSym;
   ReadParamButton->Enabled = tEnableSym;
   SymSecuStartButton->Enabled = tEnableSym;
   GetContextButton->Enabled = tEnableSym;
   //SetContextButton->Enabled = tEnableSym;
   InterceptSECUButton->Enabled = tEnableSym;

   tEnableSym = !tEnableSym;
   SymContextSecuButton->Tag = tEnableSym;

   if(tEnableSym)
   {
     SymContextSecuButton->Caption ="Stop Sym";
     StatusBar->Panels->Items[1]->Text="";
   }
   else
   {
     SymContextSecuButton->Caption ="SymContextSECU";

   }

   #define SIZE_BUF 4096*2
   char tBuf[SIZE_BUF];
   char tStr[]="@q00000762127700B402F400BE00000115 ";
   tStr[34]='\r';
   if(tEnableSym)
         tStr[3]=0x31;

   for(int tCount=0;tCount <SIZE_BUF;tCount+=35)
   {
       strncpy(&tBuf[tCount],tStr,35);
   }

   CommPort->PutBlock(tBuf,SIZE_BUF);


 }
}
//---------------------------------------------------------------------------


