//---------------------------------------------------------------------------

#ifndef LConsolH
#define LConsolH
//---------------------------------------------------------------------------

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include "CommPort.h"
#include <ExtCtrls.hpp>
#include "CSPIN.h"
#include <ComCtrls.hpp>
#include <Buttons.hpp>
#include <ImgList.hpp>
#include <Mask.hpp>
//---------------------------------------------------------------------------

#define SIZE_SECU_SENSOR_DAT 35
#define SIZE_LOGGER_ID       22
#define COMMAND_BUF_LEN      65
#define SYM_LOG_LINE_LEN     67

#define SYM_LOG_MIN_PERIOD   10
#define SYM_LOG_MAX_PERIOD   500
#define SYM_LOG_STEP_PERIOD  10
#define SYM_REFRESH_STATUS_PERIOD  200


#define GET_CONTEXT_SIZE         3
#define SET_CONTEXT_SIZE         41
#define GET_CONTEXT_ANSWER_SIZE  41


#pragma pack( push, enter_SECU3ParametersDef )
#pragma pack(1) //<--SECU3

//---------------------------------------------------------------------------

typedef struct
{
   Byte reserved[2];
   Byte stringID[SIZE_LOGGER_ID];
   Byte logFileNumber;
   Byte uartDivisor;
   Byte configFlag;
   Byte ff_fdate[2];
   Byte unused[6];

}EepromLoggedData;

typedef struct
{
 Byte hostRequest;
 Byte commandID;
 Byte data[COMMAND_BUF_LEN];
}LOGGER_PACKED;

typedef struct
{
 //  ,  0 -   
 unsigned int size;
 bool         complete;

 // -  Loggera
 LOGGER_PACKED packed;

}COMMAND;

class LogPackedSECU
{
  double  rpm;
  double  map;
  double  volt;
  double  dtoj;
  double  uoz;
  double  knock_k;
  double  knock_r;
  double  airflow;
  Byte    packByte;
// file ControlApp.cpp
//  ,          m_SensorDat.ephh_valve = (byte & (1 << 0)) != 0;
//  , m_SensorDat.carb       = (byte & (1 << 1)) != 0;
//          m_SensorDat.gas        = (byte & (1 << 2)) != 0;
//  ,           m_SensorDat.epm_valve  = (byte & (1 << 3)) != 0;
//  "CE" ,          m_SensorDat.ce_state   = (byte & (1 << 4)) != 0;

   bool shiftParametr;
   LOGGER_PACKED SymPackedSECU;

public:

  LogPackedSECU(void){Initialize(false);}
  void Initialize(bool aShiftParametr);

  char* MakeSymPackedSECU(void);
  char* GetBuffPackedSECU(void){ return ((char*)&SymPackedSECU);}
};



//---------------------------------------------------------------------------
#pragma pack( pop, enter_SECU3ParametersDef )


//---------------------------------------------------------------------------
class TConsolLoggerForm : public TForm
{
__published:	// IDE-managed Components
        TStatusBar *StatusBar;
        TGroupBox *CommPortGroupBox;
        TLabel *Label5;
        TCSpinEdit *ComCSpinEdit;
        TGroupBox *DeviceGroupBox;
        TGroupBox *ConsolGroupBox;
        TButton *GetContextButton;
        TButton *SetContextButton;
        TLabel *Label2;
        TLabel *Label3;
        TEdit *LoggerIdEdit;
        TLabel *Label7;
        TEdit *LoggerFilesCountEdit;
        TGroupBox *FlagsGroupBox;
        TLabel *Label8;
        TLabel *Label9;
        TLabel *Label10;
        TEdit *ContextLoggerFilesCountEdit;
        TEdit *ContextLoggerIdEdit;
        TGroupBox *GroupBox4;
        TGroupBox *ContextFlagsGroupBox;
        TButton *ExitButton;
        TButton *InterceptSECUButton;
        TCommPort *CommPort;
        TTimer *StatusInfoTimer;
        TBitBtn *OpenPortBitBtn;
        TEdit *ContextLoggerBaudEdit;
        TImageList *ImageList;
        TGroupBox *GroupBox1;
        TShape *TxDShape;
        TLabel *CountOutputLabel;
        TLabel *Label6;
        TGroupBox *GroupBox2;
        TShape *RxDShape;
        TLabel *CountInputLabel;
        TTimer *ClockTimer;
        TGroupBox *SymSecuGroupBox;
        TButton *SymSecuStartButton;
        TLabeledEdit *CountPackedLabeledEdit;
        TEdit *SizeEdit;
        TLabel *Label1;
        TGroupBox *SymParametrGroupBox;
        TRadioButton *SymStatRadioButton;
        TRadioButton *SymShiftRadioButton;
        TCSpinEdit *PeriodSymCSpinEdit;
        TLabel *Label12;
        TTimer *SendSymPackedTimer;
        TShape *CE_OVERRUN_Shape;
        TShape *CE_TXFULL_Shape;
        TShape *CE_RXOVER_Shape;
        TLabel *Label11;
        TLabel *Label13;
        TLabel *Label14;
        TLabel *Label15;
        TLabel *SizeDiscLabel;
        TLabel *Label16;
        TLabel *BytePerSectorLabel;
        TLabel *Label17;
        TLabel *FileSizeLabel;
        TLabel *Label18;
        TLabel *TimeWritePackedLabel;
        TLabel *Label19;
        TLabel *Label20;
        TLabel *ContextDateTimeLabel;
        TMaskEdit *DateTimeMaskEdit;
        TEdit *ClockEdit;
        TComboBox *BaudComboBox;
        TComboBox *BaudConsolComboBox;
        TLabel *Label21;
        TLabel *HandleLabel;
        TLabel *ContextTimeLabel;
        TLabel *Label4;
        TCheckBox *RequestTablesCheckBox;
        TCheckBox *ExportTablesCheckBox;
        TCheckBox *ContextRequestTablesCheckBox;
        TCheckBox *ContextExportTablesCheckBox;
        TCheckBox *BinLogCheckBox;
        TCheckBox *SmartLogCheckBox;
        TCheckBox *ContextBinLogCheckBox;
        TCheckBox *ContextSmartLogCheckBox;
        TButton *ReadWTableButton;
        TButton *WriteWTableButton;
        TButton *ReadParamButton;
        TCheckBox *SmartRequestTablesCheckBox;
        TCheckBox *ContextSmartRequestTablesCheckBox;
        TButton *SymContextSecuButton;
        TLabel *ContextCalcTimeLabel;
        void __fastcall CommPortTxDSignal(TObject *Sender);
        void __fastcall CommPortRxDSignal(TObject *Sender);
        void __fastcall CommPortDataReceived(TObject *Sender, DWORD Count);
        void __fastcall FormDestroy(TObject *Sender);
        void __fastcall CommPortRxEventSignal(TObject *Sender);
        void __fastcall GetContextButtonClick(TObject *Sender);
        void __fastcall ComCSpinEditChange(TObject *Sender);
        void __fastcall BaudComboBoxChange(TObject *Sender);
        void __fastcall TimerStatusInfoTimer(TObject *Sender);
        void __fastcall SetContextButtonClick(TObject *Sender);
        void __fastcall OpenPortBitBtnClick(TObject *Sender);
        void __fastcall ExitButtonClick(TObject *Sender);
        void __fastcall ClockTimerTimer(TObject *Sender);
        void __fastcall SymSecuStartButtonClick(TObject *Sender);
        void __fastcall SymStatRadioButtonClick(TObject *Sender);
        void __fastcall SymShiftRadioButtonClick(TObject *Sender);
        void __fastcall SendSymPackedTimerTimer(TObject *Sender);
        void __fastcall CommPortErrorSignal(TObject *Sender, int Error);
        void __fastcall InterceptSECUButtonClick(TObject *Sender);
        void __fastcall ReadWTableButtonClick(TObject *Sender);
        void __fastcall WriteWTableButtonClick(TObject *Sender);
        void __fastcall ReadParamButtonClick(TObject *Sender);
        void __fastcall SymContextSecuButtonClick(TObject *Sender);

private:	// User declarations

        TColor FNoAccessColor;
        TColor FNoEditColor;

        LogPackedSECU FPackedSymSecu;
        COMMAND FCurrentCommand;

        unsigned long FTickCount;
        int FHandleLogSecu;
        int FHandleTableSecu;

        unsigned int FContextReadCounter;
        unsigned int FCountWTablePacked;
        unsigned int FCountSymPacked;
        unsigned int FAutoGetContext;

        int FCountDataReceived;


        int FCount_TxD;
        int FCount_RxD;

        TDateTime FTimeContextSave;

        TTimer* FRxDTimer;

        void __fastcall RxDTimerTimer(TObject *Sender);
        void __fastcall CheckBufRxD(void);

        void CheckAccessSymBox(bool aOpenPort);
        void CheckAccessContextBox(bool aOpenPort);
        void CheckIndicationGroup(bool aOpenPort);
        void CheckAccessConsolBox(bool aOpenPort);
        void CheckAccessRequestButtons(bool aOpenPort);


public:		// User declarations
        __fastcall TConsolLoggerForm(TComponent* Owner);
};
//---------------------------------------------------------------------------
extern PACKAGE TConsolLoggerForm *ConsolLoggerForm;
//---------------------------------------------------------------------------
#endif

